import os

import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from lndb_rest.routers import introspection, metadata, wetlab

root_path = "/" + os.getenv("ROOT_PATH", "")

app = FastAPI(openapi_prefix=root_path)

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

app.include_router(metadata.router)
app.include_router(introspection.router)
app.include_router(wetlab.router)

if __name__ == "__main__":
    uvicorn.run("main:app", host="0.0.0.0", port=8000, root_path=root_path)



from fastapi import APIRouter
from lndb_setup import settings

router = APIRouter(prefix="/metadata")


@router.get("/")
def get_metadata():
    return {
        "name": settings.instance.name,
        "db": settings.instance.db,
        "storage_root": settings.instance.storage_root,
        "storage_region": settings.instance.storage_region,
        "_dbconfig": settings.instance._dbconfig,
        "schema_modules": settings.instance.schema,
    }



from typing import Any, Dict, List, Tuple

import lamindb as lamin
from fastapi import APIRouter
from lamindb.schema._table import table_meta
from lndb_setup import settings
from pydantic import BaseModel

router = APIRouter(prefix="/introspection")


class Column(BaseModel):
    key: str
    type: str
    primary_key: bool
    foreign_keys: List[Tuple[str, str]]
    nullable: bool
    default: Any


class TableSchema(BaseModel):
    key: str
    primary_keys: List[str]
    foreign_keys: List[Tuple[str, str]]
    columns: Dict[str, Column]


class Database(BaseModel):
    key: str
    tables: Dict[str, TableSchema]


@router.get("/", response_model=Database)
async def get_db_schema():
    schema = lamin.schema._core.get_db_metadata_as_dict()
    return schema


@router.get("/{name}")
async def get_table(key: str):
    schema_name = key.split(".")[0] if "." in key else "public"
    table_name = key.split(".")[-1]
    table_metaclass = get_table_metaclass(schema_name, table_name)

    # Rows
    table_df = lamin.select(table_metaclass).df()
    rows = table_df.to_dict(orient="records")

    # Schema
    table_object = lamin.schema._core.get_table_object(f"{schema_name}.{table_name}")
    schema = lamin.schema._core.get_table_metadata_as_dict(table_object)

    return {"schema": schema, "rows": rows}


def get_table_metaclass(schema_name, table_name):
    if settings.instance._dbconfig == "sqlite":
        return table_meta.get_model(f"{schema_name}.{table_name}")
    else:
        return table_meta.get_model(table_name)



import pandas as pd
from fastapi import APIRouter, HTTPException, UploadFile
from lnschema_wetlab.dev import parse_and_insert_df

router = APIRouter(prefix="/wetlab")


@router.post("/")
def add_samplesheet(samplesheet: UploadFile):
    df = pd.read_csv(samplesheet.file)
    try:
        added_entries = parse_and_insert_df(df, "biosample")
    except Exception as e:
        raise HTTPException(status_code=400, detail=str(e))
    return added_entries
